/******************************************************************************/
/*!
 * @file   ex_logger.h
 * @brief  外部ヘッダー
 * @note
 * @date   作成日　2021年12月21日
 * @author Ver.03 Copyright (c) Mobile Create Co., Ltd. 2016 All rights reserved.
 */
/******************************************************************************/

#ifndef _exlogger_H_
#define _exlogger_H_

#include "ex_sensor_thread.h"
#include "ex_gnss_thread.h"
#include "ex_lte_thread.h"

// ===== define =====
// ロガーflash バージョン
// !!!! 初回リリース以降、flashの構造を変更した場合、このバージョンをアップデートすること !!!!
// ログの保存情報を追加したのでバージョンをアップする
#define LOGGER_FLASH_VER		1
// TODO 個別でバージョンが必要か確認する
// 動態情報保存バージョン 保存情報変更時にバージョンをアップする
#define MLOCA_FRASH_VER			1	// ADD K.Saho 2022/11/21 圏外時動態情報の初期化処理を追加
// 一時退避ログ保存バージョン
#define TEMP_LOG_VER			2

// 日付サイズ
#define LOG_DATE_SIZE			24
// ログ文言サイズ
#define LOG_STR_SIZE			128
// ADD-S Y.Hirano 830POC-5 :FLASHログに保存した情報の中にスレッド名が存在しない
#define LOG_THREAD_NAME_SIZE	12
// ADD-E Y.Hirano 830POC-5 :FLASHログに保存した情報の中にスレッド名が存在しない
// 位置・センサー保存件数/セクター
#define POSI_SENSOR_RECORD_NUM	23		// 保存可能なサイズから計算して変更する
// ログ情報保存件数/セクター
// CHG Y.Hirano 830POC-5 :FLASHログに保存した情報の中にスレッド名が存在しない
// #define LOG_INFO_RECORD_NUM		26
#define LOG_INFO_RECORD_NUM		24
// 起動情報保存件数/セクター
#define BOOT_RECOED_NUM			170
// 動態情報保存件数/セクター
#define MLOCA_RECORD_NUM		42
// ADD-S K.Saho 2023/04/04 【機能追加】起動中のLTE通信不可時に異常ログを保存し圏内復帰時にkibanaに送る
// 一時退避ログ保存件数
#define TEMP_LOG_NUM	17
// kibanaログのcause最大文字数
#define LOG_CAUSE_MAX	180
// kibanaログ関数名最大文字数
#define LOG_FUNC_MAX	30
// ADD-E K.Saho 2023/04/04 【機能追加】起動中のLTE通信不可時に異常ログを保存し圏内復帰時にkibanaに送る

// セクター数(1ブロック当たり)
#define FLASH_SSECTOR_NUM	16
// ブロック数
#define FLASH_BLOCK_NUM		64

// ブロックサイズ
#define FLASH_BLOCK_SIZE	0x010000
// セクターサイズ
#define FLASH_SECTOR_SIZE	0x1000

// 最終ブロックオフセット
#define FLASH_BLOCK_MAX		0x3F0000

// 最終セクターオフセット
#define FLASH_SECTOR_MAX	0xF000

// 先頭セクターオフセット
#define FLASH_SECTOR_MIN	0x0000

// block1 オフセット（ログ情報、管理情報、起動情報）
#define FLASH_BLOCK1_OFFSET		0x010000

// block2 オフセット（一時退避ログ）
#define FLASH_BLOCK2_OFFSET		0x020000

// block3 オフセット（動態管理情報）
#define FLASH_BLOCK3_OFFSET		0x030000	// ADD K.Saho 2022/11/21 圏外時動態情報の初期化処理を追加

// 位置情報ブロック数
#define FLASH_BLOCK_POSI_NUM	FLASH_BLOCK_NUM -2


// 管理情報セクターオフセット
#define FLASH_SECTOR_MANAGE	FLASH_BLOCK1_OFFSET + 0x0000

// 起動情報セクターオフセット
#define FLASH_SECTOR_BOOT	FLASH_BLOCK1_OFFSET + 0x1000

// ログ情報情報セクター先頭オフセット
// CHG Y.Hirano 2022/2/25 830POC-36 :ログ情報をflashからeraseした際に、設定情報が消えているv
// #define FLASH_SECTOR_LOG	FLASH_BLOCK1_OFFSET + 0x2000
#define FLASH_SECTOR_LOG	0x2000

#if (DCOM_AGPS_OFFLINE == 1)
#define FLASH_SECTOR_AGPS_DATA	0x1000		// AGPSデータセクター先頭オフセット	// ADD M.Taguchi 2022/8/17 A-GPS AssistNow Offline対応
#endif	// #if (DCOM_AGPS_OFFLINE == 1)

// 動態情報セクターオフセット
#define FLASH_SECTOR_MLOCA		0x0000	// ADD K.Saho 2022/11/21 圏外時動態情報の初期化処理を追加
// 動態管理情報セクターオフセット
#define FLASH_MLOCA_MANAGE	FLASH_BLOCK3_OFFSET + 0x0000 // ADD K.Saho 2023/03/27 動態管理情報の保存箇所を変更
// ADD-S K.Saho 2023/04/04 圏外時動態保存のBlock跨ぎ対応
// 動態情報保存ブロック数
#define FLASH_MLOCA_BLOCK_NUM	11
// 動態情報先頭ブロックオフセット
#define FLASH_MLOCA_BLOCK_MIN	0x040000
// 動態情報最終ブロックオフセット
#define FLASH_MLOCA_BLOCK_MAX	0x0E0000
// ADD-E K.Saho 2023/04/04 圏外時動態保存のBlock跨ぎ対応

// ===== enum =====
// 保存イベント番号
typedef enum
{
	LOGGER_SAVE_NONE = 0,			// 保存データなし
	LOGGER_SAVE_PERIODEC,			// 定期保存
	LOGGER_SAVE_NUM,				// 最大値
} LOGGER_SAVE_EVENT;

// ログレベル
typedef enum
{
	FLASH_LOG_LV_NONE = 0,
	FLASH_LOG_LV_ERR,			// エラー
	FLASH_LOG_LV_WRN,			// ワーニング
	FLASH_LOG_LV_INF,			// インフォ
	FLASH_LOG_LV_DBG,			// デバッグ
} FLASH_LOG_LV;


// ===== struct =====
// ログ保存用
typedef struct
{
	FLASH_LOG_LV log_lv;						// ログレベル
	char thread_name[LOG_THREAD_NAME_SIZE];		// スレッドID		// ADD Y.Hirano 830POC-5 :FLASHログに保存した情報の中にスレッド名が存在しない
    char date[LOG_DATE_SIZE];					// 日付
    char log_str[LOG_STR_SIZE];					// ログ文字列
} LOGGER_THREAD_LOG_MSG;

typedef struct
{
	uint32_t	cpu_usage;
} CPU_USAGE_INFO;

// DEL-S D.Praveen 2022/11/28 位置・センサー情報の保存処理及び表示用デバッグコマンドのマスク
#if 0
// 位置・センサー情報保存用(1レコード)
typedef struct
{
	LOGGER_SAVE_EVENT				event;			// 保存イベント
	GnssInfo						gnss_info;		// GNSSイベント
	SENSOR_THREAD_HUMITEMP_INFO		humitemp_info;	// 温湿度センサ
    SENSOR_THREAD_IMU_INFO			imu_info;		// 加速度センサ
	CPU_USAGE_INFO					cpu_info;		// CPU使用率情報
	LTE_THREAD_MODEM_INFO			modem_info;		// モデム情報
	LTE_THREAD_LTE_LINK_INFO		ltelc_info;		// LTEコントロール情報
} LOGGER_POSI_SENSOR_RECORD;

// 位置・センサー情報保存用(1sector)
typedef struct
{
    uint16_t						crc_num;					// レコードのCRC値（正常性チェックのため）
	uint16_t						pad;						// padding
	LOGGER_POSI_SENSOR_RECORD		record[POSI_SENSOR_RECORD_NUM];	// 位置情報レコード
} LOGGER_POSI_SENSOR_SECTOR;
#endif
// DEL-E D.Praveen 2022/11/28 位置・センサー情報の保存処理及び表示用デバッグコマンドのマスク

// マネージメント情報保存用(1レコード)
typedef struct
{
	// uint16_t	crc_num;						// CRC値（正常性チェックのため）
	uint32_t	flash_ver;						// flash ROMバージョン
	uint16_t	record_save_sector;				// 位置・センサー情報保存セクター番号
	uint16_t	log_save_sector;				// ログ情報保存セクター番号
	uint32_t	record_save_block;				// 位置・センサー情報保存ブロック番号
	uint32_t	agps_end;						// AGPS実行回数
	uint16_t	temp_log_sector;				// 一時退避ログ保存セクター
	uint16_t	temp_log_ver;					// 一時退避ログ保存バージョン
} LOGGER_MANAGE_RECORD;

// マネージメント情報保存用(1sector)
typedef struct
{
	uint16_t					crc_num;			// レコードのCRC値（正常性チェックのため）
	LOGGER_MANAGE_RECORD		mng_info;			// 管理情報
} LOGGER_MANAGE_SECTOR;

// ログ情報保存用（1セクター）
typedef struct
{
	uint16_t					crc_num;							// ログのCRC値（正常性チェックのため）
	uint16_t					index;								// ログ情報の書き込みインデックス
	LOGGER_THREAD_LOG_MSG		log_record[LOG_INFO_RECORD_NUM];	// ログレコード
} LOGGER_LOG_SECTOR;

// 起動ログ
typedef struct
{
	char date[LOG_DATE_SIZE];			// 日付
} LOGGER_BOOT_RECORD;

// 起動ログ保存
typedef struct
{
	uint16_t					crc_num;						// エラーのCRC値（正常性チェックのため）
	uint16_t					index;							// 起動情報の書き込みインデックス
	LOGGER_BOOT_RECORD 			boot_record[BOOT_RECOED_NUM];	// 起動情報のレコード
}LOGGER_BOOT_SECTOR;

// ADD-S K.Saho 2022/11/21 圏外時動態情報の初期化処理を追加
// 圏外時動態情報保存用(1レコード)
typedef struct
{
	bool		unsent_flg;			// 未送信フラグ 0: 送信済, 1: 未送信 // CHG K.Saho 2022/11/24 送信フラグ -> 未送信フラグに変更
	uint8_t		numsv;				// 補足衛星数
	int16_t		p_dop;				// 位置精度劣化度
	uint32_t	mloca_id;			// 移動局ID
	int32_t		latitude;			// 緯度
	int32_t		longitude;			// 経度
	int32_t		altitude;			// 高度
	uint32_t	h_accuracy;			// 水平精度
	int32_t		speed;				// 速度
	int32_t		direction;			// 方位
	uint32_t	v_accuracy;			// 垂直精度
	uint32_t	total_distance;		// オドメータ値
	uint16_t	ttff;				// ttff
	uint16_t	agps_time;			// agps時刻
	uint16_t	agps_init;			// agpsダウンロード時間
	uint16_t	initend_ttff;		// ttffダウンロード時間
	uint16_t	year;				// 年
	uint8_t		month;				// 月
	uint8_t		day;				// 日
	uint8_t		hour;				// 時	// ADD K.Saho 2022/11/22 保存する動態情報の項目が抜けていたため修正
	uint8_t		minute;				// 分
	uint8_t		second;				// 秒
	uint8_t		rsrp;				// RSRP
	uint8_t		rsrq;				// RSRQ
	uint16_t	band;				// バンド情報
	uint8_t		gps_status;			// GPS状態
	// ADD-S K.Saho 2023/03/23 圏外時動態データの保存項目を追加
	float		temp;				// 温度
	float		humi;				// 湿度
	double		accl_x;				// X軸加速度
	double		accl_y;				// Y軸加速度
	double		accl_z;				// Z軸加速度
	// ADD-E K.Saho 2023/03/23 圏外時動態データの保存項目を追加
} LOGGER_MLOCA_RECORD;

// 圏外時動態情報保存用（1セクター）
typedef struct
{
	uint16_t				index;							// ログ情報の書き込みインデックス
	LOGGER_MLOCA_RECORD		log_record[MLOCA_RECORD_NUM];	// ログレコード
} LOGGER_MLOCA_SECTOR;
// ADD-E K.Saho 2022/11/21 圏外時動態情報の初期化処理を追加

// ADD-S D.Praveen 2023/04/21 830001-123 : pdop値を830Configからdeviceinformation.jsonに移す
typedef struct
{
	char 							m_LogLevel;					
	uint32_t 						p_pdopFilter; 				
	uint32_t 						p_haccFilter; 	
	uint32_t 						p_vaccFilter;
	char								p_ssl_expire_date[17]; 			
	char 								p_ssl_update_date[17];
} LOGGER_PARAM_RECORD;
// ADD-E D.Praveen 2023/04/21 830001-123 : pdop値を830Configからdeviceinformation.jsonに移す

// ADD-S K.Saho 2023/03/27 動態管理情報の保存箇所を変更
// 圏外時動態管理情報保存用
typedef struct
{
	uint32_t	mloca_ver;						// 動態情報保存バージョン
	uint32_t	mloca_save_block;				// 動態情報保存ブロック番号
	uint32_t	unsent_mloca_block;				// 未送信動態情報保存ブロック番号
	uint16_t	mloca_save_sector;				// 動態情報保存セクター番号
	uint16_t	unsent_mloca_sector;			// 未送信動態情報セクター番号
} LOGGER_MLOCA_MANAGE;
// ADD-E K.Saho 2023/03/27 動態管理情報の保存箇所を変更

// ADD-S K.Saho 2023/04/04 【機能追加】起動中のLTE通信不可時に異常ログを保存し圏内復帰時にkibanaに送る
// 一時退避ログ保存用(1レコード)
typedef struct
{
	char		cause[LOG_CAUSE_MAX];			// ログ文字列
	char		apl_ver[10];		// アプリバージョン
	char		modem_ver[10];		// モデムバージョン
	char		func[LOG_FUNC_MAX];			// 関数名
	int			event_id;			// イベントID
} LOGGER_TEMP_LOG_RECORD;

// 一時退避ログ保存用（1セクター）
typedef struct
{
	uint16_t				index;							// ログ情報の書き込みインデックス
	LOGGER_TEMP_LOG_RECORD		log_record[TEMP_LOG_NUM];	// ログレコード
} LOGGER_TEMP_LOG_SECTOR;
// ADD-E K.Saho 2023/04/04 【機能追加】起動中のLTE通信不可時に異常ログを保存し圏内復帰時にkibanaに送る

// prototype
extern int logger_get_manage_sector(LOGGER_MANAGE_SECTOR* pdata);
extern int logger_get_boot_sector(LOGGER_BOOT_SECTOR* pdata);
// CHG Y.Hirano 2022/2/25 830POC-36 :ログ情報をflashからeraseした際に、設定情報が消えている
// int logger_get_log_sector(LOGGER_LOG_SECTOR* pdata, uint16_t sector);
extern int logger_get_log_sector(LOGGER_LOG_SECTOR* pdata, uint32_t sector);
// DEL-S D.Praveen 2022/11/28 位置・センサー情報の保存処理及び表示用デバッグコマンドのマスク
#if 0
extern int logger_get_posi_sector(LOGGER_POSI_SENSOR_SECTOR* pdata, uint32_t block, uint16_t sector);
#endif
// DEL-E D.Praveen 2022/11/28 位置・センサー情報の保存処理及び表示用デバッグコマンドのマスク
// ADD-S M.Taguchi 2022/11/10 A-GPS AssistNow Offline対応
#if (DCOM_AGPS_OFFLINE == 1)
extern int logger_get_agps_sector(AGPS_RECORD * pdata, uint16_t sector);
extern int logger_get_agps_mng_sector(AGPS_MNG* pdata);
#endif	// #if (DCOM_AGPS_OFFLINE == 1)
// ADD-E M.Taguchi 2022/11/10 A-GPS AssistNow Offline対応
extern int logger_get_mloca_sector(LOGGER_MLOCA_SECTOR* pdata, uint32_t sector);	// CHG K.Saho 2022/11/22 動態情報読取関数の引数を修正
extern int logger_get_mloca_manage(LOGGER_MLOCA_MANAGE* pdata); // ADD K.Saho 2023/03/27 動態管理情報の保存箇所を変更
extern int logger_get_temp_log(LOGGER_TEMP_LOG_SECTOR* pdata, uint32_t sector);
extern int logger_erase_manage_sector(void);
extern int logger_erase_boot_sector(void);
extern int logger_erase_log_all(void);
// CHG Y.Hirano 2022/2/25 830POC-36 :ログ情報をflashからeraseした際に、設定情報が消えている
// int logger_erase_log_sector(uint16_t sector);
// int logger_erase_log_sector(uint32_t sector); // DEL K.Saho 2023/03/09 Cppcheckで検出された未使用関数を削除
extern int logger_erase_posi_all(void);
#if 0 // DEL-S K.Saho 2023/03/09 Cppcheckで検出された未使用関数を削除
extern int logger_erase_posi_block(uint32_t block);
extern int logger_erase_posi_sector(uint16_t sector);
#endif // DEL-E K.Saho 2023/03/09 Cppcheckで検出された未使用関数を削除
// ADD-S M.Taguchi 2022/11/10 A-GPS AssistNow Offline対応
#if (DCOM_AGPS_OFFLINE == 1)
extern int logger_erase_agps_all(void);
extern int logger_erase_agps_mng_sector(void);
#endif	// #if (DCOM_AGPS_OFFLINE == 1)
// ADD-E M.Taguchi 2022/11/10 A-GPS AssistNow Offline対応
extern int logger_erase_mloca_all(void); // ADD K.Saho 2022/11/21 圏外時動態情報の削除処理追加
extern int logger_erase_temp_log_all(void);


#endif	//_exlogger_H_
/****************************************************************************************/
/* FILE END : ex_logger.h																*/
/****************************************************************************************/
