/******************************************************************************/
/*!
 * @file   ex_lte_thread.h
 * @brief  外部ヘッダー
 * @note
 * @date   作成日　2021年12月16日
 * @author Ver.03 Copyright (c) Mobile Create Co., Ltd. 2016 All rights reserved.
 */
/******************************************************************************/
#ifndef _exltethread_H_
#define _exltethread_H_

#include <zephyr.h>
#include <modem/lte_lc.h>

// スレッドステータス
typedef enum
{
	LTE_THREAD_STS_DISCONN = 0, // LTE切断
	LTE_THREAD_STS_CONN,		// LTE接続
	LTE_THREAD_STS_NUM,			// 最大値
} LTE_THREAD_STATUS;

// flash保存用構造体
// モデム情報構造体
// https://developer.nordicsemi.com/nRF_Connect_SDK/doc/1.7.1/nrf/libraries/modem/modem_info.html
// 上記URL内のnetwork_param情報のvalue値を保持（不要なものは保持しない）
// 変数名はサイトとそろえています
// RSRPとRSRQはATコマンドより取得しています。（上記リンクとは関係ありません）
typedef struct
{
	uint16_t current_band;			// 現在のバンド
	int16_t rsrp;					// rsrp
	int16_t rsrq;					// rsrq
} LTE_THREAD_MODEM_INFO;

// ADD-S Y.Hirano 2022/2/21 830POC-34 :LTE通信ができなくなる件の暫定対処
// RSRQが0の状態が3分間継続した場合
#define LTE_THREAD_SYS_REBOOT_TIMER		180
// ADD-E Y.Hirano 2022/2/21 830POC-34 :LTE通信ができなくなる件の暫定対処

// LTEリンク構造体
// https://developer.nordicsemi.com/nRF_Connect_SDK/doc/1.7.1/nrf/libraries/modem/lte_lc.html
// 上記URLのlte_lc_conn_eval_paramsをそのまま使用。typdefで名称を他とそろえるように修正
typedef struct lte_lc_conn_eval_params LTE_THREAD_LTE_LINK_INFO;

// prototype
// CHG-S T.Itai 2022/7/28
#if 0
extern void lte_thread_create(void);
#else
extern int lte_thread_create(void);
#endif
// CHG-E T.Itai 2022/7/28
extern void lte_thread_start(void);
extern void lte_thread_lte_disconnect(void);
extern int lte_thread_rsrp_getter(void);
extern int lte_thread_rsrq_getter(void);
extern int lte_thread_band_getter(void);
extern char* lte_thread_imei_getter(void);
extern int lte_thread_antlv_getter(void);
#if 0 // DEL-S K.Saho 2023/03/09 Cppcheckで検出された未使用関数を削除
extern LTE_THREAD_MODEM_INFO lte_thread_modem_info_getter(void); // DEL K.Saho 2023/03/09 Cppcheckで検出された未使用関数を削除
extern LTE_THREAD_LTE_LINK_INFO lte_thread_ltelc_getter(void);
#endif // DEL-E K.Saho 2023/03/09 Cppcheckで検出された未使用関数を削除
extern void lte_thread_set_ref_ttlc(int64_t ttlc);
extern int lte_thread_status_get(void);		// ADD M.Taguchi 2022/10/7 830設定連動
extern bool lte_thread_conn_flg_getter(void); // ADD K.Saho 2023/04/04 【機能追加】起動中のLTE通信不可時に異常ログを保存し圏内復帰時にkibanaに送る


#endif //_exltethread_H_
/****************************************************************************************/
/* FILE END : ex_lte_thread.h															*/
/****************************************************************************************/
